''' Widgets for autocomplete chained
'''

from django import forms
from django.contrib.admin import widgets


class ChainedSelectWidget(widgets.AutocompleteSelect):
    ''' Custom widget which passes the class, css and js to the
        autocomplete filtered fields.
    '''
    def __init__(self, *args, filter_key=None, **kwargs):
        ''' Sets the class and dataset on the widget. '''
        super().__init__(*args, **kwargs)
        self.attrs['class'] = 'autocomplete-chained'
        self.attrs['data-filter-key'] = filter_key

    @property
    def media(self):
        ''' Returns the media required. '''
        return forms.Media(
            js=(
                'admin/js/vendor/jquery/jquery.js',
                'admin/js/vendor/select2/select2.full.js',
                'admin/js/jquery.init.js',
                'js/autocomplete-chained.js',
            ),
            css={
                'screen': (
                    'admin/css/vendor/select2/select2.css',
                    'admin/css/autocomplete.css',
                ),
            },
        )



''' Mixins for the admin site for autocomplete chainged.

    IMPORTANT: The mixins needs to be inherited as most left parent.
    e.g.
        class MyAdmin(FilteredFieldMixin, admin.ModelAdmin)
'''
from .widgets import ChainedSelectWidget


class ChainedSelectFieldsMixin:
    '''
        Mixin to add to admin.ModelAdmin in order to make specific fields
        an autocomplete widget linked to another dropdown.

        The configuration should be in the format:
            chained_select_fields = {
                form_field_name: linked_field_name
            }

        e.g.
            chained_select_fields = {
                'user': 'group'
            }

        Which results in the 'user' dropdown being filtered
        based on the value of the 'group' dropdown.
    '''
    chained_select_fields = dict()

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        ''' Maps the formfields to widgets if they are in the configuration
        '''
        db = kwargs.get('using')
        if db_field.name in self.chained_select_fields.keys():
            kwargs['widget'] = ChainedSelectWidget(
                db_field.remote_field, self.admin_site, using=db,
                filter_key=self.chained_select_fields[db_field.name],
            )
        else:
            return super().formfield_for_foreignkey(db_field, request, **kwargs)

        return db_field.formfield(**kwargs)


class FilteredAdminMixin:
    '''
        Mixin to add to admin.ModelAdmin in order to make fields filterable
        on the admin view interface.

        e.g.
            filtered_fields = ('user', )

        Will make it possible to filter the url
        `admin/.../user/autocomplete/?user=XXX`
    '''
    filtered_fields = tuple()

    def get_queryset(self, request):
        ''' Returns the queryset for the admin view, and filters on specified
            filtered_fields.
        '''
        queryset = super().get_queryset(request)

        # apply filters if any
        for field in self.filtered_fields:
            value = request.GET.get(field)
            if value:
                queryset = queryset.filter(**{field: value})
        return queryset



# read the contents of your README file
from os import path

from setuptools import setup

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='django-chained-autocomplete',
    version='0.0.2',
    description='Chain autocomplete dropdowns in django admin.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Framework :: Django :: 2.1',
    ],
    keywords='django autocomplete chained filtered',
    url='http://github.com/alpha1d3d/django-autocomplete-chained',
    author='Jonathan Loo',
    author_email='alpha1d3d@hotmail.com',
    license='MIT',
    packages=[
        'autocomplete_chained',
    ],
    zip_safe=False,
    python_requires='>=3.6',
    install_requires=[
        'django',
    ]
)
